%%
% This scritp file reproduces results presented in:
% 
% I. Kopriva, M. Popovi Hadija, M. Hadija, G. Aralica, "Offset-sparsity
% decomposition for automated enhancement of color microscopic image of
% stained specimen in histopathology," Journal of Biomedical Optics, Vol ?,
% pp. ??-??, 2015
%
%%

clear
close all

% OSD_rgb_JBO_2015

%% Select stained image according to the following legend:
                                                          
%%
% "1a" - H&E stained image of human liver with metastasis from colon cancer
% "2a" - H&E stained image of human liver with metabolic disorder
% "2b" - H&E stained image of human liver with obesity
% "2c" - H&E stained image of human liver with hepatocellular carcinoma (HCC), a.k.a.primary tumor
% "3a" - H&E stained image of human liver with HCC
% "3b" - CD34 stained image of human fatty liver
% "3c" - SUDAN III stained image of mouse liver
% "5a" - H&E stained image of human liver with obesity
% %5b" - H&E stained image of human liver with obesity
% "5c" - H&E stained image of human liver with obesity
% "5d" - H&E stained image of human liver with metabolic disorder
% "5i" - CD34 stained image of human liver with metabolic disorder
% "5j" - CD34 stained image of human liver with obesity
% "5k" - H&E stained image of human liver with obesity
% "5l" - H&E stained image of human liver with metastasis from colon cancer
% "5q" - H&E stained image of human liver with metabolic disorder
% "5r" - H&E stained image of human liver with metastasis from pancreatic cancer
% "5s" - H&E stained image of human liver with HCC
% "5t" - H&E stained image of human liver with HCC
% "6a" - H&E stained image of human fatty liver
% "6b" - H&E stained image of human fatty liver
% "6c" - H&E stained image of human fatty liver
% "6d" - H&E stained image of human fatyt liver
% "6i" - SUDAN III stained image of mouse liver
% "6j" - SUDAN III stained image of mouse liver
% "6k" - SUDAN III stained image of mouse liver
% "6l" - SUDAN III stained image of mouse liver
% "6q" - H&E stained image of human liver with metabolic disorder
% "6r" - H&E stained image of mouse fatty liver
% "6s" - H&E stained image of human liver with metastasis from colon cancer
% "6t" - SUDAN III stained image of mouse liver
% "7a" - H&E stained image of humnan liver with HCC
% "7b" - H&E stained image of human liver with HCC
% "7c" - H&E stained image of human liver with HCC
% "7d" - H&E stained image of human liver with metastasis from colon cancer
% "7i" - H&E stained image of human liver with metastasis from colon cancer
% "7j" - H&E stained image of human liver with metastasis from colon cancer
% "7k" - H&E stained image of human liver with metastasis from colon cancer
% "7l" - H&E stained image of human liver with metastasis from colon cancer
% "7q" - H&E stained image of human liver with metastasis from colon cancer
% "7r" - H&E stained image of human liver with metastasis from colon cancer
% "7s" - H&E stained image of human liver with metastasis from colon cancer
% "7t" - H&E stained image of human liver with metastasis from colon cancer

% select image
image_type = '3b';
 
cd .\Figures
 
G = imread(image_type,'jpg');

cd ..

% image downsampling is optional
G = double(imresize(G,0.5));

display('Image quality attributes of the original image:')
cqes=CQE(G,3,3)

% rescale image to [0, 1] interval
gm = max(max(max(G)));
G = G/gm;
[I1 I2 I3] = size(G);

% regularization constant related to optimization problem implied by Eq.(8)
lam = 1/sqrt(I1*I2);
tstart = tic;
% call osd proximal gradient method for each channel separately
for i=1:3
   [g_lr, g_sp] = proximal_gradient_osd(reshape(G(:,:,i),1,I1*I2),1/sqrt(I1*I2));
   G_osd_rgb(:,:,i)=reshape(g_sp,I1,I2);   % the enhanced image 
   G_osd_rgb_offset(:,:,i)=reshape(g_lr,I1,I2);  % the offset term
end

% rescale to maximal dynamic range
xmin=min(min(min(G_osd_rgb)));
xmin=min(xmin,0);
G_osd_rgb = G_osd_rgb - xmin;
xm=max(max(max(abs(G_osd_rgb))));
G_osd_rgb=G_osd_rgb/xm;
time_osd_rgb = toc(tstart)

% display results
figure
imagesc(G_osd_rgb)
title('OSD enhanced image')

figure
imagesc(G_osd_rgb_offset)
title('Image adapted offset')

% Store enhanced image and offset term on a disk
imwrite(G_osd_rgb,strcat(image_type,'_osd_enhanced.tiff'),'tiff', 'Compression', 'none');
imwrite(G_osd_rgb_offset,strcat(image_type,'_osd_offset.tiff'),'tiff', 'Compression', 'none');

% Estimate from the enhanced image colorfulness, sharpness and contrast 
% image quality attributes

display('Image quality attributes of the OSD enhanced image:')
G = imread(strcat(image_type,'_osd_enhanced.tiff'));
cqes=CQE(G,3,3)
